/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon 
 * 
 *	  #(@) Copyright (c) 2002, DConnect development team 
 *	  #(@) Homepage: http://www.dc.ds.pg.gda.pl/ 
 * 
 *  This  program is free software; you can redistribute it and/or 
 *  modify it under the terms of the GNU General Public License as 
 *  published by the Free Software Foundation; either version 2 of 
 *  the License. 
 * 
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the 
 *  GNU General Public License for more details. 
 * 
 *  You  should  have  received a copy  of  the GNU General Public 
 *  License  along  with  this  program;  if  not,  write  to  the 
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge, 
 *  MA 02139, USA. 
 * 
 * ---------------------------------------------------------------- 
 * }}} */ 
 
/* include local header files {{{ */ 
#include "pch.h" 
#include "main.h" 
#include "conf.h" 
#include "net.h" 
#include "misc.h" 
#include "match.h" 
#include "penalties.h" 
#include "cmd.user.h" 
/* }}} */ 
 

extern int NPENAL; /* penalties count */ 
extern int NUSERS; /* penalties count */ 
 
extern Tpenalty *penalties[MAXPENALTIES]; /* table of penalties */ 
extern struct userrec *user[MAXUSERS];	/* users table */ 
extern struct config conf;		/* configuration data */ 
 
/*  the assistant function {{{ */ 
 
/* cmd_nick2id(*usr,*nick) 
.description. 
			gets the id for the nick; if the nick is not specified or user does not exist 
			apropriate message is shown to the usr 
 
.args. 
			struct userrec *usr 	-IN-	 sending user's userec 
			char *nick				-IN-	 the nick, whose id we want 
	 
.return values. 
			<value> - the id of the user 
			-1	  - user not exists or nick not specified 
{{{ */ 
int cmd_nick2id(fd_set *fdset,struct userrec *usr, char *nick)
{ 
	int n; 
 
	if (!nick || !*nick) 
	{ 
		pubmsg(fdset,usr,"	User nick needed"); 
		pubmsg(fdset,usr,"========================================"); 
		return -1; 
	} 
	n=nick2id(nick); 
	 
	if (n<0) 
	{ 

		pubmsg(fdset,usr,"	User '%s' not found",nick); 
		pubmsg(fdset,usr,"========================================"); 
		return -1; 
	} 
	return n; 
} /* }}} */ 
	 

/* fmthln(*usr, *perm, console, visibility, *commnd, *description) 
.description. 
		displays formatted help line to >usr< whose usr->perm matches one of the >perm< 
		the line format is 
 		in chat: <HUB>#commmand		  <- decription 
  		in cons: commmand				<- decription 

.args. 
		struct userrec *usr - IN -	the user record; if usr is console user, does not display '#' before commnd 
  		char *perm			- IN - contains permissions i.e:"cas"; only users with one of thease will see the line 
		int  console		- IN - determines if line is visible from console, chat or both 
		char *commnd  		- IN - contains the command; if no specified, does not display '#' before commnd 
	 	char *description	- IN - contains description of the command; if no description is specified, display commnd as <HUB> commnd ; else display <HUB> (empty line) 

.exaple use. 
		fmthln(usr,"cas","Test",NULL);	---displays-->	'<HUB> Test' 
		fmthln(usr,"cas",NULL,"Test");	---displays-->	'					Test' 
		fmthln(usr,"cas","Axe","Test");	---displays-->	'#Axe				Test' 

{{{*/ 
void fmthln(fd_set *fdset,struct userrec *usr,char *perm, int visibility, char *commnd, char *description) 
{ 
	if ((perm) && (usr->perm) &&( 
		!(strchr(usr->perm,'s') && strchr(perm,'s')) && 
		!(strchr(usr->perm,'c') && strchr(perm,'c')) && 
		!(strchr(usr->perm,'a') && strchr(perm,'a')) )) return; 

	if (!(usr->cons && visibility&VIS_CONS) && !(!(usr->cons) && visibility&VIS_CHAT)) return;
	
	//if user is console user or no commnd is specified, display '#' before commnd 
	if (description) pubmsg(fdset,usr,(commnd)?"%c%-30s<- %s":"%c%-30s %s" ,(usr->cons==0)?((!commnd)?' ':'#'):' ',(commnd)?commnd:"",description); 
	//if no description is specified, display commnd as pubmsg; else display empty line as pubmsg 
	else pubmsg(fdset,usr,"%s",(commnd)?commnd:""); 
}/* }}} */ 
	 
 
/* penalprvmsg(*to,*from,*fmt,...) 
.description. 
	sends the private  message to from 'from' userrec 'to' userrec; 
	the message contains type of penalty and reason; 
.args. 
	struct userrec *to	//recievers's userrec 
 	struct userrec *from 	//sender's userrec 
  	char *message		//the message 
{{{ */ 
void penalprvmsg(fd_set *fdset,struct userrec *to, char *op, char *fmt, ...)
{ 
	char *str=NULL;
	va_list args;

	va_start(args , fmt);

	debug(DEBUG_PROTO,"penalprvmsg() enter");	

	str=(char *)my_vsprintf(fmt,args);
	privmsg(fdset,to,op,str);
	va_end(args);

	if ((to->penalty) && (to->penalty->reason))
 	{
		privmsg(fdset,to,op,"R e a s o n:");
		privmsg(fdset,to,op,"%s",to->penalty->reason==NULL?"NO REASON":to->penalty->reason);
	}
	my_free(str);
	debug(DEBUG_PROTO,"penalprvmsg() leave");	
} /* }}} */ 
 
/* penalmsg()
.description.
	sends the penalty messeges (pub, priv and debug)
	kicks banned users;
.args.
	Tpenalty *penrec	//info about penalty

{{{ */
void penalmsg(fd_set *fdset,Tpenalty *penrec)
{
	int i;
	long days, hours, min, sec, time_;

	debug(DEBUG_PROTO,"penalmsg() enter");	

	time_=user_exp_penalty(penrec);
	
	days=time_/(long)(24*60*60);
	time_=time_%(24*60*60);

	hours=time_/(long)(60*60);
	time_=time_%(60*60);
	min=time_/(long)60;
	time_=time_%60;

	sec=time_;

	//show info about punished nick
	pubmsg(fdset,NULL,"'%s' has no permission to: %s%s%s%s%s |<Hub> for %ld days and %ld:%ld:%ld because: %s.",
													 penrec->nick,
													 (penrec->penalty&PENALTY_BANNED)?"[enter the hub (ban)] ":"",
 													 (penrec->penalty&PENALTY_NODL)?"[download] ":"",
													 (penrec->penalty&PENALTY_NOSEARCH)?"[use search] ":"",
													 (penrec->penalty&PENALTY_NOPBLCHAT)?"[chat in public] ":"",
													 (penrec->penalty&PENALTY_NOPRVCHAT)?"[chat in private] ":"",
													 days,
													 hours,
													 min,
													 sec,
													 penrec->reason);


	
 	/*add the item to the penalties list add the penalties to userrec*/
	for (i=0;i<NUSERS;i++)
		if (user[i] && user[i]->perm && !strchr(user[i]->perm,'s') && ((penrec==user[i]->penalty) || (user[i]->ip && match(user[i]->ip,penrec->ip))))
		{
			user[i]->penalty=penrec;
			penalprvmsg(fdset,user[i],penrec->op,"You were punished as '%s'. You can't %s%s%s%s%s for %ld days and %ld:%ld:%ld.",
													 penrec->nick,
													 (penrec->penalty&PENALTY_BANNED)?"[enter the hub (ban)] ":"",
 													 (penrec->penalty&PENALTY_NODL)?"[download] ":"",
													 (penrec->penalty&PENALTY_NOSEARCH)?"[use search] ":"",
													 (penrec->penalty&PENALTY_NOPBLCHAT)?"[chat in public] ":"",
													 (penrec->penalty&PENALTY_NOPRVCHAT)?"[chat in private] ":"",
													 days,
													 hours,
													 min,
													 sec);

			debug(DEBUG_NET,"'%s' punished '%s'@%s by %s%s%s%s%s for %ld days and %ld:%ld:%ld because: %s.",
													 penrec->op,
													 user[i]->nick,
													 penrec->ip,
													 (penrec->penalty&PENALTY_BANNED)?"[ban] ":"",
 													 (penrec->penalty&PENALTY_NODL)?"[nodl] ":"",
													 (penrec->penalty&PENALTY_NOSEARCH)?"[nosearch] ":"",
													 (penrec->penalty&PENALTY_NOPBLCHAT)?"[nopblchat] ":"",
													 (penrec->penalty&PENALTY_NOPRVCHAT)?"[noprvchat] ":"",
													 days,
													 hours,
													 min,
													 sec,
													 user[i]->penalty->reason);

			if (penrec->penalty&PENALTY_BANNED) user_set_state(user[i],STATE_QUIT_SHOW);
		}
	debug(DEBUG_PROTO,"penalmsg() leave");	
} /* }}} */

/* opbreak - user with less permissions can not run a command on OP or sOP {{{ */ 
int opbreak(fd_set *fdset,struct userrec *usr,struct userrec *op, char *commnd) 
{ 
	if(!commnd) 
	{ 
		pubmsg(fdset,usr,"ERROR: in 'opbrake': command not specified; returning 1; "); 
		return 1; 
	} 
 
	if (usr->perm!=NULL && op->perm!=NULL) 
	{ 
		if (strchr(op->perm,'a') && strchr(usr->perm,'a')) 
		{ 
			pubmsg(fdset,usr,"	You are trying to %s OP", commnd);
			pubmsg(fdset,usr,"========================================"); 
			return 1; 
		} 
 
		if (strchr(op->perm,'s') && (strchr(usr->perm,'a') || strchr(usr->perm,'s'))) 
		{ 
		pubmsg(fdset,usr,"	You are trying to %s sOP", commnd);
		pubmsg(fdset,usr,"========================================"); 
		pubmsg(fdset,op,"%s is trying to %s you.", usr->nick,commnd);
		return 1; 
		} 
 
		if (strchr(op->perm,'a') && strchr(usr->perm,'s')) 
		{ 
   		pubmsg(fdset,usr,NULL,"You are %sing OP", commnd);
		pubmsg(fdset,usr,"========================================");
		return 0; 
		} 
	} 
	 
	return 0; 
 
}/* }}} */ 
	 
/* }}} */ 
/* the COMMAND functions {{{ */ 
 
/* cmdhelp - shows usr the help screen {{{*/ 
void cmd_help(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	fmthln(fdset,usr,NULL,  VIS_ALL,	"========================================",	NULL); 
	fmthln(fdset,usr,NULL,  VIS_ALL,  "HELP",	NULL); 
	fmthln(fdset,usr,NULL,  VIS_ALL,  "========================================", NULL); 

	fmthln(fdset,usr,NULL,  VIS_ALL,	"[commands listed by categories]",	NULL); 
	fmthln(fdset,usr,NULL,  VIS_ALL,  NULL,NULL); 

	fmthln(fdset,usr,NULL,  VIS_ALL,	"[info]:",	NULL); 
	fmthln(fdset,usr,NULL,  VIS_ALL,	"-----------------------------------------", NULL); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"conf",			"shows current hub configuration"); 
	fmthln(fdset,usr,"c",	  VIS_CONS,	"stat",			"shows statistics");
	fmthln(fdset,usr,NULL,  VIS_ALL,	"uptime",		"shows hub uptime"); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"users",		"shows all hub users"); 
	fmthln(fdset,usr,NULL,  VIS_ALL,	"version",		"shows version of the hub"); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"who",			"shows who is on the console"); 
	fmthln(fdset,usr,"sac", VIS_ALL,	"whois <nick>",	"shows info about user"); 
	fmthln(fdset,usr,"sac", VIS_ALL,	"whoip <ip>","shows nicks of every user having specified ip"); 
	fmthln(fdset,usr,NULL,  VIS_ALL,	"-----------------------------------------", NULL); 
	fmthln(fdset,usr,"as",  VIS_ALL,  NULL,NULL); 

	fmthln(fdset,usr,"as",  VIS_ALL,	"[punishments]:", NULL); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"-----------------------------------------",NULL); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"amnesty <ip>",					"removes penalty from ip"); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"amnesty_all",					"clears the punished ip's list"); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"lspunished",					"shows the list of punished ip's"); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"punish <nick> <how> [how long in minutes] [reason]","punishes the user");
	fmthln(fdset,usr,"as",  VIS_ALL,	NULL,							"<how>=(nosearch,nopblchat,noprvchat,nodl,ban)"); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"kick <nick> <reason>", 		"disconnects the user"); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"-----------------------------------------",NULL); 
	fmthln(fdset,usr,"as",  VIS_ALL,  NULL,NULL); 

	fmthln(fdset,usr,"as",  VIS_ALL,	"[communication & redir]:" ,NULL); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"-----------------------------------------" ,NULL); 
	fmthln(fdset,usr,"as",  VIS_ALL,	"msg <nick> <message>"		,"sends message to user in pub chat as hub"); 
	fmthln(fdset,usr,"s",   VIS_ALL,	"msg_all <message>"		,"sends message to all users in pub chat as hub"); 
	fmthln(fdset,usr,NULL,  VIS_ALL,	"mmsg <nick> <message>"		,"sends message to user in pub chat as your message"); 
	fmthln(fdset,usr,NULL,  VIS_ALL,	"mmsg_all <message>"		,"sends message to all users in pub chat as your message"); 
	fmthln(fdset,usr,NULL,  VIS_CONS,	"pubchat"					,"toggles public chat mode"); 


	fmthln(fdset,usr,"as",  VIS_ALL,	"redirect <nick> <where>"	,"moves user to another hub"); 
	fmthln(fdset,usr,"s",   VIS_ALL,	"redirect_all <where>"		,"moves all users to another hub"); 
	fmthln(fdset,usr,"sa",  VIS_ALL,  "-----------------------------------------" ,NULL); 
	fmthln(fdset,usr,"sa",  VIS_ALL,  NULL ,NULL); 
	
	fmthln(fdset,usr,"c",   VIS_CONS,	"[other commands]:" ,NULL); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"-----------------------------------------" ,NULL); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"rehash"	,"reloads hub configuration"); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"quit"		,"leave the console"); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"jq"		,"toggles displaing of joins & quits traffic"); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"boost"		,"toggles info-booster"); 
	fmthln(fdset,usr,"c",   VIS_CONS,	"-----------------------------------------" ,NULL); 
	return; 

} /* }}} */ 
 
/* hlp_redirect -  shows help for redirect command {{{*/ 
void hlp_redirect(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Description: "); 
	pubmsg(fdset,usr,"	Moves the user to another hub."); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Syntax:"); 
	pubmsg(fdset,usr,"	%credirect <nick> <where>",(usr->cons)?' ':'#'); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"	<nick> is the nick of user"); 
	pubmsg(fdset,usr,"	<where> is the adress of hub into <user> should be moved"); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
/* cmdredirect - redirect's the user to another hub {{{ */ 
void cmd_redirect(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	int n; 
	char *tmp1=NULL, *tmp2=NULL; 
	char *__strtok_temp__=NULL;

	//checking if user has apriopriate permissions
	if (usr->perm && !strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return;

	if (!line || !*line) { hlp_redirect(fdset, usr );  return;} 	

	tmp1=strtok_r(line," ",&__strtok_temp__); //who 
	tmp2=strtok_r(NULL,"|",&__strtok_temp__);//where 
 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   REDIRECT"); 
	pubmsg(fdset,usr,"========================================"); 

	// checks the nick, returns id
	n=cmd_nick2id(fdset,usr,tmp1); if(n<0)return; 

	if (!tmp2) 
	{ 
		pubmsg(fdset,usr,"	Do not know where to redirect"); 
		pubmsg(fdset,usr,"========================================"); 
		return; 
	} 
 
	if (!strcmp(usr->nick,tmp1)) 
	{ 
		pubmsg(fdset,usr,"	You are trying to move yourself"); 
		pubmsg(fdset,usr,"========================================"); 
		return; 
	} 
 
	if (opbreak(fdset,usr,user[n],"redirect"))return; 
		 
	debug(DEBUG_NET,"'%s'(OP) moved %s to '%s'",usr->nick,user[n]->nick,tmp2); 
 
	sendf(fdset,user[n],"$ForceMove %s|",tmp2); 
	user_set_state(user[n],STATE_QUIT_SHOW);
	pubmsg(fdset,usr,"	Redirecting '%s' to %s",tmp1,tmp2); 
	pubmsg(fdset,usr,"========================================"); 
 
	if((n=nick2id(tmp1))>=0) user_set_state(user[n],STATE_QUIT_SHOW);
} /* }}} */ 
 
/* hlp_redirect -  shows help for redirect command {{{*/ 
void hlp_redirect_all(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Description: "); 
	pubmsg(fdset,usr,"	Moves all the users to another hub."); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Syntax:"); 
	pubmsg(fdset,usr,"	%credirect_all <where>",(usr->cons)?' ':'#'); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"	<where> is the adress of hub into users should be moved."); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
/* cmdredirectall - redirect all users to anther hub {{{*/ 
void cmd_redirect_all(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	int temp=0,n; 
	char *tmp1=NULL,*__strtok_temp__;

	//checking if user has apriopriate permissions
	if (usr->perm && !strchr(usr->perm,'s')) return; 

	if (!line || !*line) {hlp_redirect_all(fdset, usr );  return;} 		 
	tmp1=strtok_r(line,"| ",&__strtok_temp__); //<where> 
	 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   REDIRECT_ALL"); 
	pubmsg(fdset,usr,"========================================"); 
 

	if (!tmp1) 
	{ 
		pubmsg(fdset,usr,"	do not know where to redirect to"); 
	  	pubmsg(fdset,usr,"========================================"); 
		return; 
	} 
 
	debug(DEBUG_NET,"'%s'(sOP) moved ALL to '%s'",usr->nick,tmp1); 
 
	pubmsg(fdset,usr,"	Moving users to %s",tmp1); 
 
	temp=0; 
	for(n=0;n<NUSERS;n++) 
		if (user[n] && user_tst_state(user[n],STATE_LOGGEDIN|STATE_SR|STATE_REGISTERED) && usr->perm && !strchr(user[n]->perm,'s'))
		{
			temp++; 
			sendf(fdset,user[n],"$ForceMove %s|",tmp1); 
			user_set_state(user[n],STATE_QUIT_SHOW);
		} 
 
	pubmsg(fdset,usr,"	Communicate sent to %d of %d users",temp,NUSERS); 
	pubmsg(fdset,usr,"========================================"); 
	debug(DEBUG_NET,"'%s'(OP) moved ALL to '%s'",usr->nick,tmp1); 
} /* }}} */ 
 
/* cmdpassword {{{*/ 
void cmd_password(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   PASSWORD"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	Command not supported yet"); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
/* cmduptime - displays hub uptime {{{ */ 
void cmd_uptime(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   UPTIME"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	%s",uptime(starttime)); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
/* cmdversion - displays hub version {{{ */ 
void cmd_version(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   VERSION"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	%s (build: %s)",PACKAGE_VERSION,BUILD_TIME); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
 	/* cmdlspunished - displays the list of punished {{{*/ 
void cmd_lspunished(fd_set *fdset, struct userrec *usr) 
{ 
	int n;
	long time_, days, hours, min, sec;
	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return; 

	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   LSPUNISHED"); 
	pubmsg(fdset,usr,"========================================"); 
 
	if (NPENAL==0) pubmsg(fdset,usr,"	Nobody is punished :("); 
	else pubmsg(fdset,usr,"	IP : Nick: Punisher : Penalty : Time(d,hh:mm:ss) : Reason");
 
	for(n=0;n<NPENAL;n++) 
	{
		
		time_=user_exp_penalty(penalties[n]);
		if (time_<=0) continue;
	
		days=time_/(long)(24*60*60);
		time_=time_%(24*60*60);
		
		hours=time_/(long)(60*60);
		time_=time_%(60*60);
		
		min=time_/(long)60;
		time_=time_%60;
		
		sec=time_;
		
		pubmsg(fdset,usr,"	%s : %s : %s  : %s%s%s%s%s : %ld, %ld:%ld:%ld : %s",  penalties[n]->ip,
																penalties[n]->nick,
																strchr(usr->perm,'s')?penalties[n]->op:"HUB",
														  		(penalties[n]->penalty&PENALTY_BANNED)?"[ban] ":"",
														  		(penalties[n]->penalty&PENALTY_NODL)?"[nodl] ":"",
																(penalties[n]->penalty&PENALTY_NOSEARCH)?"[nosearch] ":"",
																(penalties[n]->penalty&PENALTY_NOPBLCHAT)?"[nopblchat] ":"",
																(penalties[n]->penalty&PENALTY_NOPRVCHAT)?"[noprvchat] ":"",
																days,
																hours,
																min,
																sec,
																penalties[n]->reason); 
	}
	pubmsg(fdset,usr,"========================================");  
} /* }}} */ 


/* hlp_punish - displays help for punish command {{{*/ 
void hlp_punish(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr,"	Punishment not specified"); 
	pubmsg(fdset,usr,"	Available punishments sample use:"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"	%cpunish <nick> <how> [time] [reason]",(usr->cons)?' ':'#');
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"	<how>"); 
	pubmsg(fdset,usr,"	nosearch		- user has no permission to use search"); 
	pubmsg(fdset,usr,"	nopblchat		- user has no permission to chat in main chat"); 
	pubmsg(fdset,usr,"	noprvchat		- user has no permission to chat in private chat"); 
	pubmsg(fdset,usr,"	nochat			- user has no permission to	chan in main and private chat"); 
	pubmsg(fdset,usr,"	nodl			- user has no permission to download"); 
	pubmsg(fdset,usr,"	ban				- user is banned"); 
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"	You can punish with multiple punishments, merging them with '+'");
	pubmsg(fdset,usr,"	i.e. ");
	pubmsg(fdset,usr,"		%cpunish UserNick nochat+nodl+nosearch 20",(usr->cons)?' ':'#');
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"	[time]			- the duration of punishment in minutes");
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"	[time] and [reason] are optional");
	pubmsg(fdset,usr," 	if no time is specified the duration of penalty will be %ld minute(s)",STD_PENALTY_DURATION);
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"	1 day - 1440 minutes");
	pubmsg(fdset,usr,"	3 days - 4320 minutes");
	pubmsg(fdset,usr,"	1 week - 10080 minutes");
	pubmsg(fdset,usr,"	eternity (1 year) - -1 minutes :)");
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"	examples:");
	pubmsg(fdset,usr,"		%cpunish UserNick nochat 10 Was naughty.",(usr->cons)?' ':'#');
	pubmsg(fdset,usr,"		%cpunish UserNick nochat+nodl",(usr->cons)?' ':'#');
	pubmsg(fdset,usr,"		%cpunish UserNick nosearch Was naughty.",(usr->cons)?' ':'#');
	pubmsg(fdset,usr,"========================================");
} /* }}} */ 
 

/* cmdpunish - punishes the user with one of the avaliable penalties {{{ */ 
void cmd_punish (fd_set *fdset, struct userrec *usr,char *line) 
{ 
	const char *punish[]= 
	{ 
		"nopblchat", 
		"ban", 
		"nosearch", 
		"noprvchat", 
		"nodl", 
		"nochat", 
		"" /* indicates end of list */ 
	}; 
// 	long days, hours, min, sec, time;
	long /*i=0,*/ n=0,countpunish=0, duration=0;
	char *who_tmp=NULL,*how_tmp=NULL,*msg_tmp=NULL,*tmp=NULL,*tmp1=NULL,*pun=NULL, *__strtok_temp__;
	int punishment=0;
 
	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s') && !strchr(usr->perm,'a'))return; 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"   PUNISH"); 
	pubmsg(fdset,usr,"========================================"); 

	if (!line || !*line)
	{ 
		hlp_punish(fdset,usr); 
		return;
	}

	who_tmp=strtok_r(line," ",&__strtok_temp__); //who 
	how_tmp=strtok_r(NULL," ",&__strtok_temp__); //how 
	tmp=strtok_r(NULL,"",&__strtok_temp__); //time + message 

	if (tmp)
	{
		duration=strtol(tmp,&tmp1,10);

		if (duration==0) duration=conf.std_penalty_duration;
		if (*tmp1) msg_tmp=tmp1;
	}
	else duration=conf.std_penalty_duration;
	
	if (!who_tmp || !how_tmp)
	{
		hlp_punish(fdset,usr) ;
		return;
	} 
 
	n=cmd_nick2id(fdset,usr,who_tmp);  
	if(n<0)return; 
	
	if (duration==-1) duration=365*24*60; //-1 is one year
	 
 	duration=duration*60;// to get duration in seconds

	pun=strtok_r(how_tmp,"+",&__strtok_temp__);

	while(pun && *pun)
	{
		/* parse punishment {{{ */
		countpunish=-1;
		while(*punish[++countpunish]) // was ...nish++], and didn't work
			if (pun && !strcasecmp(pun,punish[countpunish])) break;
		how_tmp=pun; 								/* tmp2 contains current punishment(s) */
		pun=strtok_r(NULL,"+",&__strtok_temp__);

     	switch(countpunish)
		{
			case 0: //nopblchat
					punishment|=(int)PENALTY_NOPBLCHAT;
					continue; 
 
			case 1: //ban 
					punishment|=(int)PENALTY_BANNED;	
					continue;

			case 2: //nosearch 
					punishment|=(int)PENALTY_NOSEARCH;
					continue;
 
			case 3: //noprvchat 
					punishment|=(int)PENALTY_NOPRVCHAT;
					continue;
 
			case 4: //nodl 
					punishment|=(int)PENALTY_NODL;
					continue;
 
			case 5: //nochat 
					punishment|=(int)PENALTY_NOPRVCHAT|PENALTY_NOPBLCHAT;
					continue;
 
			default: 
					pubmsg(fdset,usr,"Unknown punishment '%s'. Type '#punish' for more help",how_tmp);
					break;
		} 
/* end of parse punishment }}}*/	
	}

	if (punishment) 
	{				
		if ( usr->perm && strchr(user[n]->perm,'s')) return;
		user_set_penalty(fdset,user[n]->ip,user[n]->nick,usr->nick, punishment,(unsigned long)time(NULL),duration,msg_tmp,0);
	}

	pubmsg(fdset,usr,"========================================"); 

} /* }}} */ 

/* hlp_msg -  shows help for msg command {{{*/ 
void hlp_msg(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	MSG"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Description: "); 
	pubmsg(fdset,usr,"	Shows the <message> to the <user> as 'Hub'"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Syntax:"); 
	pubmsg(fdset,usr,"	%cmsg <nick> <message>",(usr->cons)?' ':'#'); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
	 
/* cmdmsg - sends message as hub to the user {{{*/ 
void cmd_msg(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	int n; 
	char *tmp1=NULL,*tmp2=NULL; 
	char *__strtok_temp__=NULL;

	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return; 
	if (!line || !*line) {hlp_msg(fdset,usr); return;} 
	tmp1=strtok_r(line," ",&__strtok_temp__); //who 
	tmp2=strtok_r(NULL,"",&__strtok_temp__); //message 
 
	n=cmd_nick2id(fdset,usr,tmp1); if(n<0)return; 
 
	if (!tmp2) 
	{ 
		pubmsg(fdset,usr,"========================================"); 
		pubmsg(fdset,usr,"	MSG"); 
		pubmsg(fdset,usr,"========================================"); 
		pubmsg(fdset,usr,"	You have to specify message"); 
		pubmsg(fdset,usr,"========================================"); 
		return; 
	} 
	if (user[n]!=usr) pubmsg(fdset,user[n],tmp2); 
}/* }}} */ 

/* hlp_msg -  shows help for msg command {{{*/ 
void hlp_mmsg(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	MMSG"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Description: "); 
	pubmsg(fdset,usr,"	Shows the <message> to the <user> as Your message"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Syntax:"); 
	pubmsg(fdset,usr,"	%cmmsg <nick> <message>",(usr->cons)?' ':'#'); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
	 
/* cmdmsg - sends message as hub to the user {{{*/ 
void cmd_mmsg(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	int n; 
	char *tmp1=NULL,*tmp2=NULL; 
	char *__strtok_temp__=NULL;

	//checking if user has apriopriate permissions
	if (!line || !*line) {hlp_msg(fdset,usr); return;} 
	tmp1=strtok_r(line," ",&__strtok_temp__); //who 
	tmp2=strtok_r(NULL,"",&__strtok_temp__); //message 
 
	n=cmd_nick2id(fdset,usr,tmp1); if(n<0)return; 
 
	if (!tmp2) 
	{ 
		pubmsg(fdset,usr,"========================================"); 
		pubmsg(fdset,usr,"	MSG"); 
		pubmsg(fdset,usr,"========================================"); 
		pubmsg(fdset,usr,"	You have to specify message"); 
		pubmsg(fdset,usr,"========================================"); 
		return; 
	} 
	if (user[n]!=usr) sendf(fdset,user[n],"<%s> %s|",usr->nick,tmp2); 
}/* }}} */ 

/* hlp_pubchat - sends message as hub to the user {{{*/ 
void hlp_pubchat(fd_set *fdset, struct userrec *usr) 
{ 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	PUBCHAT"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Description: "); 
	pubmsg(fdset,usr,"	enables for a console user chating freely"); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"Syntax:"); 
	pubmsg(fdset,usr,"	%cpubchat",(usr->cons)?' ':'#'); 
	pubmsg(fdset,usr," "); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
	 
/* cmd_mmsg - sends message as usr to the user {{{*/ 
void cmd_pubchat(fd_set *fdset, struct userrec *usr,char *line) 
{ 
	//checking if user has apriopriate permissions
	if (!usr->cons) return;
	
	if (usr->cons&2) 
	{
		pubmsg(fdset,usr,"Chating in public for you is now off");
		usr->cons=1;
	}
	else 
		{
			usr->cons=3;
			pubmsg(fdset,usr,"Chating in public for you is now on");
			pubmsg(fdset,usr,"To stop. write 'pubchat' one more time");
		}	
}/* }}} */ 
	 
/* hlp_msg_all -  shows help for msg command {{{*/ 
void hlp_msg_all(fd_set *fdset, struct userrec *usr) 
{
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	MSG_ALL");
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"Description: ");
	pubmsg(fdset,usr,"	Shows the <message> to all users as 'Hub'");
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"Syntax:");
	pubmsg(fdset,usr,"	%cmsg_all <message>",(usr->cons)?' ':'#');
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"========================================");
} /* }}} */

/* cmdmsgall - sends message as hub to all users {{{*/ 
void cmd_msg_all(fd_set *fdset, struct userrec *usr,char *line) 
{
	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s')) return; 
	if (!line|| !*line) {hlp_msg_all(fdset,usr); return;} 
 
	pubmsg(fdset,NULL,line); 
	debug(DEBUG_CHAT,"<HUB>(%s) -> ALL: %s", usr->nick,line);
	//DEBUGMSG 
} /* }}} */ 

/* hlp_msg_all -  shows help for msg command {{{*/ 
void hlp_mmsg_all(fd_set *fdset, struct userrec *usr) 
{
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	MMSG_ALL");
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"Description: ");
	pubmsg(fdset,usr,"	Shows the <message> to all users as your message");
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"Syntax:");
	pubmsg(fdset,usr,"	%cmmsg_all <message>",(usr->cons)?' ':'#');
	pubmsg(fdset,usr," ");
	pubmsg(fdset,usr,"========================================");
} /* }}} */

/* cmdmsgall - sends message as hub to all users {{{*/ 
void cmd_mmsg_all(fd_set *fdset, struct userrec *usr,char *line) 
{
	//checking if user has apriopriate permissions
	if (!line|| !*line) {hlp_msg_all(fdset,usr); return;} 
 
	sendf(fdset,NULL,"<%s> %s|",usr->nick,line); 

	//DEBUGMSG 
} /* }}} */ 
 
/* cmd_amnesty - cancels all of current user's penalties {{{ */ 
void cmd_amnesty (fd_set *fdset, struct userrec *usr,char *line) 
{ 
	char *tmp1=NULL; 
	char *__strtok_temp__=NULL;
	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return; 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	AMNESTY"); 
	pubmsg(fdset,usr,"========================================"); 
	if (!line|| !*line) return;
	tmp1=strtok_r(line," ",&__strtok_temp__); //ip 
 
	if (!tmp1) 
	{ 
		pubmsg(fdset,usr,"	User's nick or ip needed");
		pubmsg(fdset,usr,"========================================");	 
	return; 
	} 
 
	user_set_penalty(fdset,tmp1,tmp1,"",PENALTY_NONE,0,0,NULL,0);
	
	pubmsg(fdset,usr,"	Amnesty of USER/IP: %s",tmp1);
	pubmsg(fdset,usr,"========================================"); 
	//DEBUGMSG 
} /* }}} */ 

 /* cmdamnestyall - cancels all of penalties {{{ */
void cmd_amnesty_all (fd_set *fdset, struct userrec *usr) 
{
	int i,max=NPENAL;

	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s')) return;
	
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	AMNESTY_ALL");
	pubmsg(fdset,usr,"========================================");
	
	for(i=0; i<max;i++) delpenalty(0);
	penalties_write();

	pubmsg(fdset,usr,"	Amnesty of all punished");
	pubmsg(fdset,usr,"========================================");
	//DEBUGMSG 	
} /* }}} */ 
 
	 
/* cmd_conf {{{ */ 
void cmd_conf (fd_set *fdset, struct userrec *usr) 
{ 
	//can be used only from console
	if (!usr->cons) return; 

	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	CONF"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"Config file: %s", conf.conf_main); 
	pubmsg(fdset,usr,"Welcome file: %s", conf.conf_welcome); 
	pubmsg(fdset,usr,"MOTD file: %s", conf.conf_motd); 
	pubmsg(fdset,usr,"Bans file: %s", conf.conf_banned); 
	pubmsg(fdset,usr,"Console users file: %s", conf.conf_cusers); 
	pubmsg(fdset,usr,"Console allow file: %s", conf.conf_callow); 
	pubmsg(fdset,usr,"Nicks allow file: %s", conf.conf_nallow); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"Log level: %u", conf.log_level); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"Log files dir: %s", conf.log_dir); 
	pubmsg(fdset,usr,"Main log: %s", conf.log_main); 
	pubmsg(fdset,usr,"Network access log: %s", conf.log_net); 
	pubmsg(fdset,usr,"Protocol log: %s", conf.log_proto); 
	pubmsg(fdset,usr,"Warnings log: %s", conf.log_wrng); 
	pubmsg(fdset,usr,"Console log: %s", conf.log_con); 
	pubmsg(fdset,usr,"Public chat log: %s", conf.log_chat); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"Main listening port: %u", conf.listen_port_main); 
	pubmsg(fdset,usr,"Console listening port: %u", conf.listen_port_cons); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"UID: %u", conf.uid); 
	pubmsg(fdset,usr,"GID: %u", conf.gid); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"Hub Name: %s", conf.hubname); 
	pubmsg(fdset,usr,"Admin contact: %s", conf.admin_contact); 
	pubmsg(fdset,usr,"User limit: %u", conf.userlimit); 
	pubmsg(fdset,usr,"Allow broken keys: %d", conf.allow_broken_key); 
	pubmsg(fdset,usr,"Ping timeout: %u", conf.ping_timeout); 
	pubmsg(fdset,usr,"Registration timeout: %u", conf.register_timeout); 
	pubmsg(fdset,usr,"Send retries: %u", conf.send_retries); 
	pubmsg(fdset,usr,"Send retries time(us): %u", conf.send_retries_time); 
	pubmsg(fdset,usr,"penalties update interval (s): %u", conf.penalties_update_interval); 
	pubmsg(fdset,usr,"Minimum data shared: %llu MB", conf.minshare); 
	pubmsg(fdset,usr,"Minimum slots: %u", conf.minslots); 
	pubmsg(fdset,usr,"Allow chat: %d", conf.allow_chat); 
	pubmsg(fdset,usr,"Flags: %u", conf.flags); 
	pubmsg(fdset,usr,"========================================"); 

	if (conf.redirect_switch)
	{
	    pubmsg(fdset,usr,"Redirections:");
	    if(conf.redirect_switch&REDIRECT_SWITCH_ACCESS)      pubmsg(fdset,usr,"Access:      %s",conf.redirect_access);
	    if(conf.redirect_switch&REDIRECT_SWITCH_HUB_IS_FULL) pubmsg(fdset,usr,"Hub is full: %s",conf.redirect_hub_is_full);
	    if(conf.redirect_switch&REDIRECT_SWITCH_MINSHARE)    pubmsg(fdset,usr,"Min_share:   %s",conf.redirect_minshare);
	    if(conf.redirect_switch&REDIRECT_SWITCH_MINSLOTS)    pubmsg(fdset,usr,"Min_slots:   %s",conf.redirect_minslots);
	    pubmsg(fdset,usr,"========================================"); 
	}
	
	
} /* }}} */

/* cmd_rehash {{{ */
void cmd_rehash(fd_set *fdset, struct userrec *usr)
{
	//can be used only from console
	if (!usr->cons) return;

	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	REHASH");
	pubmsg(fdset,usr,"========================================");
	finish(SIGHUP);
} /* }}} */

/* cmd_stat {{{ */
void cmd_stat (fd_set *fdset, struct userrec *usr)
{
	long long int tmp=0;
	int i, reg=0;

	//can be used only from console
	if (!usr->cons) return;

	for(i=0;i<NUSERS;i++) 
	    if (user_tst_state(user[i],STATE_REGISTERED)) 
	    {
		tmp+=user[i]->share;
		reg++;
	    }

	tmp=tmp/(int)(1024*1024); 
	
	i=0; // MB
	if (tmp>=1024)
	{
		tmp=tmp/(int)1024; 
		i=1; // GB

		if (tmp>=1024) 
		{ 
		    tmp=tmp/(int)1024; 
		    i=2; // TB
		}
	}

	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	STAT");
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	Users registered: %d",reg);
	pubmsg(fdset,usr,"	Users waiting:    %d",NUSERS-reg);
	pubmsg(fdset,usr,"	Shared data: %llu%cB",tmp,(i==0)?'M':((i==1)?'G':'T'));
	pubmsg(fdset,usr,"	Uptime: %s",uptime(starttime));
	pubmsg(fdset,usr,"========================================");
} /* }}} */


/* cmd_whois {{{ */ 
void cmd_whois (fd_set *fdset, struct userrec *usr,char *line) 
{ 
	int x,count; 
	long long tmp;

	//checking if user has apriopriate permissions
	if (usr->perm && !strchr(usr->perm,'c') && 
		!strchr(usr->perm,'s') &&
		!strchr(usr->perm,'a')) return; 

	x=cmd_nick2id(fdset,usr,line); 
		 
	if (x<0) return; 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	WHOIS"); 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"Nick: %s",user[x]->nick); 
	pubmsg(fdset,usr,"IP: %s",user[x]->ip); 
	pubmsg(fdset,usr,"Mode: %s",(user[x]->flag==1)?"online":((user[x]->flag==3)?"away":"unknown")); 
	pubmsg(fdset,usr,"Idle: %s",uptime(user[x]->ping)); 

	if (user[x]->cons==0) 
	{ 
		tmp=user[x]->share; 
		tmp/=1024*1024; count=0; // MB 
		if (tmp>=1024) 
	{ 
		tmp/=1024; count=1; // GB 
		if (tmp>=1024) { tmp/=1024; count=2; } // TB 
	} 
	pubmsg(fdset,usr,"Shared: %llu%cB",tmp,(count==0)?'M':((count==1)?'G':'T')); 
	} 
 
	pubmsg(fdset,usr,"E-mail: %s",user[x]->email); 
	pubmsg(fdset,usr,"Connection: %s",user[x]->ctype); 
	pubmsg(fdset,usr,"Description: %s",user[x]->desc); 
	pubmsg(fdset,usr,"Permissions: %s", usr->perm?user[x]->perm:"not set"); 
	 
	if (user[x]->penalty) pubmsg(fdset,usr,"Punishments: %d",user[x]->penalty->penalty); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 

/* cmd_whois {{{ */ 
void cmd_whoip (fd_set *fdset, struct userrec *usr,char *line) 
{ 
	int i,count=0; 

	//checking if user has apriopriate permissions
	if (usr->perm && !strchr(usr->perm,'c') && 
		!strchr(usr->perm,'s') &&
		!strchr(usr->perm,'a')) return; 

	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	WHOIP"); 
	pubmsg(fdset,usr,"========================================"); 

	if (!line) 
	{
		pubmsg(fdset,usr,"You have to specify user's ip");
		pubmsg(fdset,usr,"========================================"); 
		return;
	}
	
	for(i=0;i<NUSERS;i++)
		if (user[i] && 
			user_tst_state(user[i],STATE_REGISTERED) && 
			!strcmp(user[i]->ip,line)) 
			{
				pubmsg(fdset,usr,"%s",user[i]->nick);
				count++;
			}
	
	if (!count) pubmsg(fdset,usr,"%s","Nobody of this ip was found");
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 

/* cmd_boost {{{ */ 
void cmd_boost (fd_set *fdset, struct userrec *usr) 
{ 
	//can be used only from console
	if (!usr->cons) return; 

	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	BOOST"); 
	pubmsg(fdset,usr,"========================================"); 

	toggle_flag(fdset,&conf,usr,INFO_BOOST); 
} /* }}} */ 
 
/* cmd_jq {{{ */ 
void cmd_jq (fd_set *fdset, struct userrec *usr) 
{ 
	//can be used only from console
	if (!usr->cons) return; 

	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	JQ"); 
	pubmsg(fdset,usr,"========================================"); 
 
	toggle_flag(fdset,&conf,usr,SHOW_JQ); 
} /* }}} */ 
 
/* cmd_quit {{{ */ 
void cmd_quit (fd_set *fdset, struct userrec *usr) 
{ 

	//can be used only from console
	if (!usr->cons) return; 
	if ( usr->perm && !strchr(usr->perm,'c')) return; 

	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	QUIT"); 
	pubmsg(fdset,usr,"========================================"); 
	user_set_state(usr,STATE_QUIT_SHOW); 
} /* }}} */ 
 
 
/* cmd_who {{{ */ 
void cmd_who (fd_set *fdset, struct userrec *usr) 
{
	int x;
	if (!usr->cons) return; 
	if ( usr->perm && !strchr(usr->perm,'c')) return; 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	WHO"); 
	pubmsg(fdset,usr,"========================================"); 

	pubmsg(fdset,usr,"USER				 FROM			 IDLE"); 
	for (x=0;x<NUSERS;x++) 
		if (user[x] && user_tst_state(user[x],STATE_REGISTERED) &&(user[x]->cons)) 
			sendf(fdset,usr,"%-20s %-15s  %s\r\n",user[x]->nick,user[x]->ip,uptime(user[x]->ping)); 
	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
/* cmd_users {{{ */ 
void cmd_users (fd_set *fdset, struct userrec *usr) 
{ 
	long i,tmp=0;
	 
	if (!usr->cons) return; 
 
	if ( usr->perm && !strchr(usr->perm,'c')) return; 
 
	pubmsg(fdset,usr,"========================================"); 
	pubmsg(fdset,usr,"	USERS"); 
	pubmsg(fdset,usr,"========================================"); 
 
	for (i=0;i<NUSERS;i++) 
	{ 
		if (user[i] && user_tst_state(user[i],STATE_LOGGEDIN|STATE_REGISTERED))  
		{ 
			pubmsg(fdset,usr,"%-26s",user[i]->nick);  
			tmp=(tmp+1)%4; 
			
			if (!tmp) pubmsg(fdset,usr," ");  
		}
 
	}
	 
	if (tmp)  
	{
		pubmsg(fdset,usr," ");  
		tmp=0; 
	} 

	pubmsg(fdset,usr,"========================================"); 
} /* }}} */ 
 
 
/* cmd_kick {{{ */
void cmd_kick (fd_set *fdset, struct userrec *usr,char *line)
{
	char *tmp1;
	int i;
	char *__strtok_temp__=NULL;
	//checking if user has apriopriate permissions
	if ( usr->perm && !strchr(usr->perm,'s') && !strchr(usr->perm,'a')) return;

	if (!line||!*line)  return;

	tmp1=strtok_r(line," ",&__strtok_temp__);
	pubmsg(fdset,usr,"========================================");
	pubmsg(fdset,usr,"	KICK: %s",tmp1);
	pubmsg(fdset,usr,"========================================");

	i=cmd_nick2id(fdset,usr,tmp1); 
	
	if(i<0)return;

	if (user[i]==usr) 
	{
		pubmsg(fdset,usr,"	It's simplier to use 'quit'.");
		pubmsg(fdset,usr,"========================================");
	   	return;
	}

	if (opbreak(fdset,usr,user[i],"kick"))return;
		
	tmp1=strtok_r(NULL,"",&__strtok_temp__);
	pubmsg(fdset,user[i],"You have been kicked off by %s.",usr->nick);
	if (tmp1) pubmsg(fdset,user[i],"Reason: %s.",tmp1);

	debug(DEBUG_STD,"Kicked off %s@%s by %s.",user[i]->nick,user[i]->ip,usr->nick);
	user_set_state(user[i],STATE_QUIT_SHOW);
	
} /* }}} */



/* 
void command(*usr,*line) 
 
.description: 
  runs the in main chat or console given command, depending on usr->cons 
 
.args: 
	struct userrec *usr  - userrec of the user, who gives the command 
	char *line - the line following the command 
	 
{{{*/ 
 
void command(fd_set *fdset, struct userrec *usr,char *line) 
{ 
/* const decl. {{{ */ 
	const char *cmd[]= 
	{ 
		"#help",			//0 
		"#password", 
		"#uptime", 
		"#redirect", 
		"#redirect_all", 
		"#version",			//5 
		"#lspunished", 
		"#punish", 
		"#amnesty", 
		"#msg",				 
		"#msg_all", 		//10
		"#amnesty_all", 
		"#conf",			 
		"#rehash", 
		"#quit", 
		"#stat", 			//15
		"#whois",			 
		"#who", 
		"#users", 
		"#kick", 			//20
		"#jq",				 
 		"#boost", 
		"#mmsg",
		"#mmsg_all",
		"#pubchat",			//25
		"#whoip",
 		"" /* indicates end of list */ 
	}; 
 
	const char *sep=" "; 
 /* }}} */ 
  
	int cmdcount=0; 
	char *cmdtmp=NULL; //stores the command 
	char *cmdparam=NULL; //stores the parameters 
	char *line_copy=NULL; //stores the copy of line 
 	char *__strtok_temp__=NULL;
/* parase the line {{{*/ 

	my_duplicate(line,&line_copy);
 
	//get the command 
	cmdtmp=strtok_r(line_copy,sep,&__strtok_temp__);  
	if (!usr->cons) cmdtmp=strtok_r(NULL,sep,&__strtok_temp__);  
 
	//get the parameters of the command 
	cmdparam=strtok_r(NULL,"",&__strtok_temp__); 
	
	cmdcount=-1; 
	if (cmdtmp && (*cmdtmp=='#'||usr->cons) && *cmdtmp!=0) 
	{ 
		cmdcount=0; 
		while(*cmd[cmdcount]) 
		{ 
			if (!strcasecmp(cmdtmp,cmd[cmdcount]+(usr->cons?1:0))) break; 
			else cmdcount++; 
		} 
		if (!usr->cons && *cmd[cmdcount]) sendf(fdset,usr,"%s|",line); 
	} 
 
/* to uncomment add // at beggining 
//DEBUG PURPOSES {{{ 
	pubmsg(fdset,usr,"{+ + + + + + + + + + + + + + + + + + + + + + + + + + + + }"); 
  	pubmsg(fdset,usr,"	cons: %d; perm: %s;",usr->cons,usr->perm); 
  	pubmsg(fdset,usr,"	line: '%s'",line); 
  	pubmsg(fdset,usr,"	command: '%s'; parameters:'%s'",cmdtmp,cmdparam); 
	pubmsg(fdset,usr,"{+ + + + + + + + + + + + + + + + + + + + + + + + + + + + }"); 
//DEBUG PURPOSES }}} */ 
 
/* }}} */ 
 
	switch (cmdcount) 
	{ 
		case CMD_HELP			:cmd_help(fdset,usr,cmdparam);		goto leave; 
		case CMD_PASSWORD		:cmd_password(fdset,usr,cmdparam); 	goto leave; 
		case CMD_UPTIME			:cmd_uptime(fdset,usr); 				goto leave; 
		case CMD_REDIRECT		:cmd_redirect(fdset,usr,cmdparam); 	goto leave; 
		case CMD_REDIRECT_ALL	:cmd_redirect_all(fdset,usr,cmdparam);goto leave; 
		case CMD_VERSION		:cmd_version(fdset,usr); 				goto leave; 
		case CMD_LSPUNISHED		:cmd_lspunished(fdset,usr);			goto leave; 
		case CMD_PUNISH			:cmd_punish(fdset,usr,cmdparam);		goto leave; 
		case CMD_MSG			:cmd_msg(fdset,usr,cmdparam);			goto leave; 
		case CMD_MSG_ALL 		:cmd_msg_all(fdset,usr,cmdparam);		goto leave; 
		case CMD_AMNESTY 		:cmd_amnesty(fdset,usr,cmdparam);		goto leave; 
		case CMD_AMNESTY_ALL 	:cmd_amnesty_all(fdset,usr);			goto leave; 
		case CMD_CONF			:cmd_conf(fdset,usr);					goto leave; 
		case CMD_REHASH			:cmd_rehash(fdset,usr);				goto leave; 
		case CMD_QUIT			:cmd_quit(fdset,usr);					goto leave; 
		case CMD_STAT			:cmd_stat(fdset,usr);					goto leave; 
		case CMD_WHOIS			:cmd_whois(fdset,usr,cmdparam);		goto leave; 
		case CMD_WHO			:cmd_who(fdset,usr);					goto leave; 
		case CMD_USERS			:cmd_users(fdset,usr);				goto leave; 
		case CMD_KICK			:cmd_kick(fdset,usr,cmdparam);		goto leave; 
		case CMD_JQ				:cmd_jq(fdset,usr);					goto leave; 
		case CMD_BOOST			:cmd_boost(fdset,usr);				goto leave; 
		case CMD_MMSG			:cmd_mmsg(fdset,usr,cmdparam);		goto leave; 
		case CMD_MMSG_ALL 		:cmd_mmsg_all(fdset,usr,cmdparam);	goto leave; 
		case CMD_PUBCHAT		:cmd_pubchat(fdset,usr,cmdparam);		goto leave; 
	 	case CMD_WHOIP			:cmd_whoip(fdset,usr,cmdparam);		goto leave; 
		
		default: 
			if(user_tst_penalty(usr,PENALTY_NOPBLCHAT) || (usr->cons && !(usr->cons&2))) goto leave; 
 
			if(conf.allow_chat) 
 			{ 
				if (cmdtmp && *cmdtmp=='#') sendf(fdset,usr,"%s|",line); 
				else 
					if (usr && usr->cons&2) 
					{
						sendf(fdset,NULL,"<%s> %s|",usr->nick,line);
						debug(DEBUG_CHAT,"<%s> %s",usr->nick,line); 					
					}
					else
						{
							sendf(fdset,NULL,"%s|",line); 
							debug(DEBUG_CHAT,"%s",line); 
						}
			}
			else pubmsg(fdset,usr,"Public chat is not allowed."); 
	} 
 
leave: 
	my_free(line_copy); 
} 
/*}}}*/ 
