/* $Id: net.c,v 1.91 2005/01/05 12:35:42 rav Exp $ */

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon
 * 
 *	  #(@) Copyright (c) 2002, DConnect development team
 *	  #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* include local header files {{{ */
#include "pch.h"
#include "main.h"
#include "misc.h"
#include "console.h"
/* }}} */

extern struct userrec *user[];
extern int NUSERS;
extern struct config conf;
extern int maxfd;

void my_send(fd_set *fdset,struct userrec *usr, char *msg)
{
	int counter=0,
		msglen=0,
		n=0,
		tries=conf.send_retries;
		int where=0;

	struct timeval timeout;
	
//	debug(DEBUG_PROTO,"    my_sendf() enter");

	msglen=strlen(msg);

//	debug(DEBUG_PROTO,"1");
	
	while (!user_tst_state(usr,STATE_QUIT) && tries-- && counter<msglen)
	{
		timeout.tv_sec=0;
		timeout.tv_usec=conf.send_retries_time;

		FD_SET(usr->sock,fdset);
		if(select(maxfd,NULL,fdset,NULL,&timeout)<0)
		{
			if (errno!=EINTR) debug(DEBUG_WRNG,"select() [WRITE] failed: %s",strerror(errno));	
			where=1;
			continue;
		}
	
		if FD_ISSET(usr->sock,fdset)
		{
			n=send( usr->sock, msg+counter, msglen-counter, 0 );
		
	 		if (n==-1)
			{
				where=0;
				if (errno!=EAGAIN) break;
				continue;
			}
			counter+=n;
		}

	}

//	debug(DEBUG_PROTO,"2");
	FD_CLR(usr->sock,fdset);

	if (counter<msglen)
	{
//		debug(DEBUG_PROTO,"3");

		if (tries!=-1) debug(DEBUG_WRNG,"%s failed %s@%s: %s",where?"select()":"send()",usr->nick?usr->nick:"Unknown",usr->ip,strerror(errno));	
		else debug(DEBUG_WRNG,"my_send() failed %s@%s: retries limitation reached.",usr->nick?usr->nick:"Unknown",usr->ip,strerror(errno));	
		user_set_state(usr,STATE_QUIT_SHOW);	
	}

//	debug(DEBUG_PROTO,"    my_sendf() leave");
}

/* sendf() - something like printf(), but to a socket {{{ */
void sendf( fd_set *fdset, struct userrec * usr, const char * fmt, ... )
{
	va_list args;
	int i=0;
	char *str=NULL;


//	debug(DEBUG_PROTO,"    sendf() enter");
	
	va_start( args, fmt );
//	debug(DEBUG_PROTO,"     vsprintf() enter");
	str=(char *)my_vsprintf(fmt,args);
//	debug(DEBUG_PROTO,"     vsprintf() leave");
	va_end( args );
	
	if (usr) 
	{
//		debug(DEBUG_PROTO,"1");
		my_send(fdset, usr, str); 
//		debug(DEBUG_PROTO,"2");
		debug(DEBUG_PROTO,"      SENT %s '%s' ",usr->nick?usr->nick:"Unknown",str);
	}
	else
		{ 
//			debug(DEBUG_PROTO,"2");

			/* multicasting only to non-console users */
			for ( i = 0; i < NUSERS; i++ )
				if (user[i] && user_tst_state(user[i],STATE_REGISTERED) && 
					user[i]->cons==0) my_send(fdset,user[i], str); 

			debug(DEBUG_PROTO,"      SENT 2 ALL '%s'",str);			
		}
	my_free(str);
//	debug(DEBUG_PROTO,"    sendf() leave");
} /* }}} */




/* this is required, since after disconnect a socket is switched to a TIME_WAIT
 * state and the listening port remains occupied for several minutes */
/* setsockopts() {{{ */
char setsockopts(int sock,unsigned char reuse) 
{
	int flag=1;
	char ret=0;
	struct linger linger;

	if (reuse) 
	{
		linger.l_onoff=0; linger.l_linger=0;
		if (setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&flag,sizeof(flag))<0) ret=1;
		if (setsockopt(sock,SOL_SOCKET,SO_LINGER,&linger,sizeof(linger))<0) ret=1;
	}

	if (setsockopt(sock,SOL_SOCKET,SO_KEEPALIVE,&flag,sizeof(flag))<0) ret=1;
	return ret;
} /* }}} */


/* pubmsg(*to,*fmt,...)
 	.description.
		send message as "Hub" to to's public chat with fmt format

	.args.
		struct userrec *to 	- IN -	the user record; if to==NULL - send the message to all users
  		char *fmt			- IN - "line format"
		char ...			- IN - additional params

	.sample use.
		string="me"
		pubmsg(usr,"It's %s",string); // displays "it's me" to the user
		pubmsg(NULL,"Have no fear"); // displayse "Have no fear" to all users
	
 {{{ */

void pubmsg(fd_set *fdset, struct userrec *to, const char *fmt, ...)
{
	char *str=NULL;
	va_list args;

	debug(DEBUG_PROTO,"  pubmsg() enter");

	va_start( args, fmt );
	str=(char *)my_vsprintf(fmt,args);
	va_end(args);
	
	if (to && to->cons) sendf(fdset,to,"%s\r\n",str);
	else sendf(fdset,to,"<HUB> %s|",str);
	
	my_free(str);

	debug(DEBUG_PROTO,"  pubmsg() leave");
}
/* }}} */

/* privmsg(*to,*from,*fmt,...)
 	.description.
		send private message as 'from->nick'

	.args.
		struct userrec *to 	- IN -	reciever's user record; if to==NULL - sends the priv message to all users (not tested yet)
		struct userrec *from- IN -	sender's user record; if from==NULL - sends the message as <HUB>
  		char *fmt			- IN - "line format"
		char ...			- IN - additional params
{{{ */

void privmsg(fd_set *fdset,struct userrec *to, char *from, const char *fmt, ...)
{
	int n;
	char *str=NULL, *frm=NULL;
	va_list args;
	va_start( args, fmt );

	debug(DEBUG_PROTO,"  privmsg() enter");


	str=(char *)my_vsprintf(fmt,args);

	va_end(args);

	if (!from) my_duplicate("HUB",&frm);
	else my_duplicate(from,&frm);

	if (!to) 
	{	
		for (n=0;n<NUSERS;n++) 
			if (user[n])
			{	
				if(!user[n]->cons) sendf(fdset, user[n],"$To: %s From: %s $<%s> %s|",user[n]->nick,frm,user[n]->nick,str);
				else sendf(fdset,user[n],"PM %s: %s",frm,str);
			}
	}
	else 
		if (!to->cons) sendf(fdset,to,"$To: %s From: %s $<%s> %s|",to->nick,frm,frm,str);
		else sendf(fdset,to,"PM %s: %s",frm,str);

  	my_free(frm);	
	my_free(str);
	debug(DEBUG_PROTO,"  privmsg() leave");
} /* }}} */


/* 	
	int my_recv() - receives data from socket {{{
	it is used for getting the message from the socket
	returns 
	0	- there is an error
	1	- everything went fine 
*/
int my_recv(struct userrec *usr, char **message)
{
	char *buf=NULL;
	int count=0, n=128, pos=0;
	

	/* read all available data */
	buf=(char *)my_malloc(n);
	
	while(1) 
	{
		count=recv(usr->sock,buf+pos,n-pos,0); //get the text
		if (count>0) pos+=count;
		if (((count>0)&&(pos<n))||((count<0)&&(errno==EAGAIN))) break;
	
		if (count<0 && errno!=EAGAIN) 
		{ 
#if defined(WIN32) || defined(__CYGWIN__)
			if (user_tst_state(usr,STATE_LOGGEDIN|STATE_REGISTERED)) user_set_state(usr,STATE_QUIT_SHOW);
			else user_set_state(usr,STATE_QUIT);
	
			goto leave; 
#else
			debug(DEBUG_WRNG,"recv() failed from %s@%s: %s.",(usr->nick)?usr->nick:"Unknown",usr->ip,strerror(errno)); 
			break; 
#endif
		}

		if (count==0) 
		{ 
			if (user_tst_state(usr,STATE_LOGGEDIN|STATE_REGISTERED)) user_set_state(usr,STATE_QUIT_SHOW);
			else  user_set_state(usr,STATE_QUIT);
	
			goto leave; 
		}

		n*=2;
		buf=my_realloc(buf,n);
		if (buf==NULL) 
		{ 
			if (user_tst_state(usr,STATE_LOGGEDIN|STATE_REGISTERED)) user_set_state(usr,STATE_QUIT_SHOW);
			else  user_set_state(usr,STATE_QUIT);

			goto leave; 
		}
	}

	if (pos<=0) goto leave;
	buf=my_realloc(buf,pos+1);
	buf[pos]=0;

	/* update last ping */
	if (user_tst_state(usr,STATE_REGISTERED)) usr->ping=time(NULL);

	*message=buf;

	return 1;
leave:
		my_free(buf);
		
		return 0;
}

/* }}}*/

/* disconnect() - disconnect a user with optional reason message {{{ */
void disconnect(fd_set *fdset,struct userrec *usr,char *reason) 
{
	debug(DEBUG_PROTO,"disconnect() enter");	
	debug(DEBUG_NET,"Disconnected %s@%s: %s",(usr->nick)?usr->nick:"Unknown",usr->ip,(reason)?reason:"Connection closed.");
	if (reason) pubmsg(fdset,usr,reason);
	close(usr->sock);
	my_free(usr->ip);
	my_free(usr->password);
	my_free(usr->buf);
	my_free(usr->nick);
	my_free(usr->con_ip);
	my_free(usr->ver);
	my_free(usr->desc);
	my_free(usr->ctype);
	my_free(usr->email);
	my_free(usr->perm);
	my_free(usr);
	debug(DEBUG_PROTO,"disconnect() leave");
} /* }}} */

/* strip_telnet() - strip telnet codes - from eggdrop {{{ */
void strip_telnet(struct userrec *usr,char *buf,int *len) 
{
	unsigned char *p = (unsigned char *) buf, *o = (unsigned char *) buf;
	int mark;

	debug(DEBUG_PROTO,"strip_telnet() enter");	
	while (*p != 0) 
	{
		while ((*p != TLN_IAC) && (*p != 0))
		*o++ = *p++;
		if (*p == TLN_IAC) 
		{
			p++;
			mark = 2;
			if (!*p) mark = 1;		/* bogus */
			if ((*p >= TLN_WILL) && (*p <= TLN_DONT)) 
			{
				mark = 3;
				if (!*(p + 1)) mark = 2;		/* bogus */
	  		}
	  		
			if (*p == TLN_WILL) 
			{
				/* WILL X -> response: DONT X */
				/* except WILL ECHO which we just smile and ignore */
				if (*(p + 1) != TLN_ECHO) 
				{
					write(usr->sock, TLN_IAC_C TLN_DONT_C, 2);
					write(usr->sock, p + 1, 1);
				}
	  		}

			if (*p == TLN_DO) 
			{
				/* DO X -> response: WONT X */
				/* except DO ECHO which we just smile and ignore */
				if (*(p + 1) != TLN_ECHO) 
				{
					write(usr->sock, TLN_IAC_C TLN_WONT_C, 2);
					write(usr->sock, p + 1, 1);
				}
			}

			if (*p == TLN_AYT) 
			{
				/* "are you there?" */
				/* response is: "hell yes!" */
				write(usr->sock, "\r\nHell, yes!\r\n", 14);
			}

			/* Anything else can probably be ignored */
			p += mark - 1;
			*len = *len - mark;
		}
	}

	*o = *p;
	
	debug(DEBUG_PROTO,"strip_telnet() leave");	
} /* }}} */

/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */
